#ifndef _JZ_P32MX_I2C_H_
#define _JZ_P32MX_I2C_H_
#include <stdint.h>

#if defined(_USE_SFRS_BASE_ADDR)
  extern const void * _sfrs_base_addr_0;
  #define _i2c_base_addr        (_sfrs_base_addr_0 + 0x5000)
#else
  #define _i2c_base_addr	((const void *)0xBF805000)
#endif

#if defined(__32MX1XX__) || defined(__32MX2XX__)
  #define I2C1		0
  #define I2C2		1
#elif defined(__32MX3XX__) || defined(__32MX4XX__)
  #define I2C1		0
  #define I2C2		2
#elif defined(__32MX5XX__) || defined(__32MX6XX__) || defined(__32MX7XX__)
  #define I2C1		3
  #define I2C2		4
  #define I2C3		0
  #define I2C4		1
  #define I2C5		2
#endif

  #define _I2C_ADDR(x)	(_i2c_base_addr + (x)*0x100)

  /* x=0: I2C1 x=1: I2C2 */
  #define I2C_CON(x)          (*(volatile uint16_t *)(_I2C_ADDR(x) + 0x00))
  #define I2C_CON_CLR(x)      (*(volatile uint16_t *)(_I2C_ADDR(x) + 0x04))
  #define I2C_CON_SET(x)      (*(volatile uint16_t *)(_I2C_ADDR(x) + 0x08))
  #define I2C_CON_INV(x)      (*(volatile uint16_t *)(_I2C_ADDR(x) + 0x0C))

  #define I2C_STAT(x)         (*(volatile uint16_t *)(_I2C_ADDR(x) + 0x10))
  #define I2C_STAT_CLR(x)     (*(volatile uint16_t *)(_I2C_ADDR(x) + 0x14))
  #define I2C_STAT_SET(x)     (*(volatile uint16_t *)(_I2C_ADDR(x) + 0x18))
  #define I2C_STAT_INV(x)     (*(volatile uint16_t *)(_I2C_ADDR(x) + 0x1C))

  #define I2C_ADD(x)          (*(volatile uint16_t *)(_I2C_ADDR(x) + 0x20))
  #define I2C_ADD_CLR(x)      (*(volatile uint16_t *)(_I2C_ADDR(x) + 0x24))
  #define I2C_ADD_SET(x)      (*(volatile uint16_t *)(_I2C_ADDR(x) + 0x28))
  #define I2C_ADD_INV(x)      (*(volatile uint16_t *)(_I2C_ADDR(x) + 0x2C))

  #define I2C_MSK(x)          (*(volatile uint16_t *)(_I2C_ADDR(x) + 0x30))
  #define I2C_MSK_CLR(x)      (*(volatile uint16_t *)(_I2C_ADDR(x) + 0x34))
  #define I2C_MSK_SET(x)      (*(volatile uint16_t *)(_I2C_ADDR(x) + 0x38))
  #define I2C_MSK_INV(x)      (*(volatile uint16_t *)(_I2C_ADDR(x) + 0x3C))

  #define I2C_BRG(x)          (*(volatile uint16_t *)(_I2C_ADDR(x) + 0x40))
  #define I2C_BRG_CLR(x)      (*(volatile uint16_t *)(_I2C_ADDR(x) + 0x44))
  #define I2C_BRG_SET(x)      (*(volatile uint16_t *)(_I2C_ADDR(x) + 0x48))
  #define I2C_BRG_INV(x)      (*(volatile uint16_t *)(_I2C_ADDR(x) + 0x4C))

  #define I2C_TRN(x)          (*(volatile uint8_t  *)(_I2C_ADDR(x) + 0x50))
  #define I2C_TRN_CLR(x)      (*(volatile uint8_t  *)(_I2C_ADDR(x) + 0x54))
  #define I2C_TRN_SET(x)      (*(volatile uint8_t  *)(_I2C_ADDR(x) + 0x58))
  #define I2C_TRN_INV(x)      (*(volatile uint8_t  *)(_I2C_ADDR(x) + 0x5C))

  #define I2C_RCV(x)          (*(volatile uint8_t  *)(_I2C_ADDR(x) + 0x60))

/* not yet
 * #define I2C_DD(x)          (*(volatile uint8_t  *)(_I2C_ADDR(x) + 0x60))
 *( ADD(4) : 5120 )
 *( ADD(5) : 5220 )
 *   DD(3) : 5320
 *   DD(4) : 5420
 *   DD(5) : 5020
  #define I2C1		3
  #define I2C2		4
  #define I2C3		0
  #define I2C4		1
  #define I2C5		2
 */
    #define I2C_CON_SEN                    0x00
    #define I2C_CON_RSEN                   0x01
    #define I2C_CON_PEN                    0x02
    #define I2C_CON_RCEN                   0x03
    #define I2C_CON_ACKEN                  0x04
    #define I2C_CON_ACKDT                  0x05
    #define I2C_CON_STREN                  0x06
    #define I2C_CON_GCEN                   0x07
    #define I2C_CON_SMEN                   0x08
    #define I2C_CON_DISSLW                 0x09
    #define I2C_CON_A10M                   0x0A
    #define I2C_CON_STRICT                 0x0B
    #define I2C_CON_SCLREL                 0x0C
    #define I2C_CON_SIDL                   0x0D
    #define I2C_CON_ON                     0x0F
    #define I2C_CON_IPMIEN                 0x0B
    #define I2C_CON_I2CSIDL                0x0D
    #define I2C_CON_I2CEN                  0x0F

    #define I2C_STAT_TBF                   0x00
    #define I2C_STAT_RBF                   0x01
    #define I2C_STAT_R_W                   0x02
    #define I2C_STAT_S                     0x03
    #define I2C_STAT_P                     0x04
    #define I2C_STAT_D_A                   0x05
    #define I2C_STAT_I2COV                 0x06
    #define I2C_STAT_IWCOL                 0x07
    #define I2C_STAT_ADD10                 0x08
    #define I2C_STAT_GCSTAT                0x09
    #define I2C_STAT_BCL                   0x0A
    #define I2C_STAT_TRSTAT                0x0E
    #define I2C_STAT_ACKSTAT               0x0F
    #define I2C_STAT_I2CPOV                0x06

#endif /* _JZ_P32MX_I2C_H_ */
